# include <math.h>

//constant-------------------------------------------------------------
const double h=1.054571596e-27;
const double c=2.99792458e10;
const double pi=3.14159265359;
const double me=9.10938188e-28;
const double e=4.80274e-10;
const double kB=1.3806503e-16;
const double alpha_fine=e*e/h/c;
const double I=me*pow(e,4)/h/h/2.;
const double freq1=I/2./pi/h;
const double T_ion=I/kB;
const double a_St_B=pi*pi*pow(kB,4)/60./c/c/pow(h,3);
const double sigma_t=(8.*pi/3)*pow(e,4)/pow(c,4)/pow(me,2);
const double aB_2=pow(h,4)/pow(e,4)/pow(me,2);

//---------------------------------------------------------------------
double T_Hyd(int i)
{
        if(i)
        return -T_ion/(double)i/(double)i;
        else
        return 0.;
}

//math function--------------------------------------------------------
double Hav(double x)
{
        double res=0.0;
        if(x>=0.0)
                res=1.;
        return res;
}


double ln_Factor(double numb)
{
     double res=0.;
     for(double i=1;i<=numb;i++)
     res+=log(i);
     return res;
}

double hgf_Pol_Inh(double alpha,double beta,double gamma,
double z,double inhibitor)
{
        double var, var_a,var_b,var_g, var_a_abs, var_b_abs, var_g_abs;
        double Summ;
	int sum_limit;
	double sign1,sign=1.;
        var=0.;
        Summ=exp(inhibitor);
	sum_limit=int(fabs(alpha*(alpha>beta)+beta*(alpha<=beta))+0.1);
        for(int i=1;i<=sum_limit;i++)
        {
				var_a=alpha+(double)i-1.;
				var_b=beta+(double)i-1.;
				var_g=gamma+(double)i-1.;

				var_a_abs=fabs(var_a);
				var_b_abs=fabs(var_b);
				var_g_abs=fabs(var_g);


                var+=log(var_a_abs)+log(var_b_abs)
			+log(fabs(z))-log(var_g_abs)-log((double)i);

				sign1=1.;
				if(var_a_abs)
				sign1*=(var_a/var_a_abs);
				if(var_b_abs)
				sign1*=(var_b/var_b_abs);
				if(var_g_abs)
				sign1*=(var_g/var_g_abs);
				if(z)
				sign1*=(z/fabs(z));

				sign*=sign1;

                Summ+=sign*exp(var+inhibitor);

        }
        return Summ;
}

//---------------------------------------------------------------------

//spontaneous transition-----------------------------------------
double reduced_matr_2(double n1,double l1,double n2,double l2)
{

     double res,var,var1;
     double nu,nd,l;
     if((l1-l2)*(l1-l2)!=1)
     return 0;
     l=l1*(l1>l2)+l2*(l2>l1);
     if(l==0)
     return 0;
     nu=n1*(l1>l2)+n2*(l2>l1);
     nd=n2*(l1>l2)+n1*(l2>l1);

     var1=(4*nu*nd/(nu-nd)/(nu-nd));
     var=0.5*log(l)-log(4.)-ln_Factor(2*l-1)+0.5*
     (ln_Factor(nu+l)+ln_Factor(nd+l-1)-ln_Factor(nu-l-1)-ln_Factor(nd-l))
     +(l+1)*log(4*nu*nd)+(nu+nd-2*l-2)*log(fabs(nu-nd))-(nu+nd)*log(nu+nd);
     res=(hgf_Pol_Inh(-nu+l+1,-nd+l,2*l,-var1,var)-pow(fabs(nu-nd)/(nu+nd),2)*
     hgf_Pol_Inh(-nu+l-1,-nd+l,2*l,-var1,var));
     return res*res;
}

double A_exact(double i,double k)
{
        double res,freq=freq1*(1./k/k-1./i/i),var=0.;
        if(i<=k+0.1)
	return 0.;	
        res=4./3.*pow(2.*pi*freq,3.)/h/c/c/c*(h*h/e/me)*(h*h/e/me);
        for(double l1=0.;l1<=i-0.99;l1++)
	for(double l2=0.;l2<=k-0.99;l2++)
	{
    	  var+=reduced_matr_2(i,l1,k,l2);
	}
	res*=var/i/i;
	return res;
}


//Johnson's approximation, 1972
double gaunt_bb(double i,double k)
{
      double var=1./(1.-(k/i)*(k/i));
	  double kk=1./k;
	  double g0,g1,g2;
      g0=1.133*(k==1)+1.0785*(k==2)+(k>=3)*
		  (0.9935+0.2328*kk-0.1296*kk*kk);
	  g1=-0.4059*(k==1)-0.2319*(k==2)-(k>=3)*
		  (0.6282-0.5598*kk+0.5299*kk*kk)*kk;
	  g2=0.07014*(k==1)+0.02947*(k==2)+(k>=3)*
		  (0.3887-1.181*kk+1.47*kk*kk)*kk*kk;
     return g0+g1*var+g2*var*var;
}

double A_Kramers(double i,double k)
{
        double res,freq;
        if(i<=k+0.1)
	return 0.;	
        res=pow(2.,6)*e*e*I*I/pow(3.,1.5)/pi/me/c/c/c/h/h;
        freq=(1./k/k-1./i/i);
        res/=(freq*pow(i,5)*pow(k,3));
        res*=gaunt_bb(i,k);
        return res;        
}

//---------------------------------------------------------------------
//ionization cross section---------------------------------------------

//Johnson's approximation, 1972
double gaunt_ion(double k,double freq)
{
          double var=freq1/k/k/freq;
	  double kk=1./k;
	  double g0,g1,g2;
      g0=1.133*(k==1)+1.0785*(k==2)+(k>=3)*
		  (0.9935+0.2328*kk-0.1296*kk*kk);
	  g1=-0.4059*(k==1)-0.2319*(k==2)-(k>=3)*
		  (0.6282-0.5598*kk+0.5299*kk*kk)*kk;
	  g2=0.07014*(k==1)+0.02947*(k==2)+(k>=3)*
		  (0.3887-1.181*kk+1.47*kk*kk)*kk*kk;
     return g0+g1*var+g2*var*var;
}

double ion(double i,double freq)
{
        double res=me*pow(e,10)/pow(3.,1.5)/pi/pi/c/pow(h,6);
        res*=Hav(freq-freq1/i/i)/(pow(i,5)*pow(freq,3));
        res*=gaunt_ion(i,freq);
        return res;
}


//---------------------------------------------------------------------
//equilibrium radiation------------------------------------------------

double fill0(double freq,double T)
{
        return 1./(exp(2.*pi*h*freq/kB/T)-1.);
}

double plank(double freq,double T)
{
        return (8.*pi)*(freq*freq/c/c/c)*fill0(freq,T);
}


//---------------------------------------------------------------------


//free electron statsum------------------------------------------------

double g(double Te)
{
        return pow((me*kB*Te/2./pi),3./2.)/h/h/h;
}

//---------------------------------------------------------------------



//ionization rate---------------------------------------------------

double ion_rate(double i,double T)
{
        double res=0.0,freq_i=freq1/i/i,freq_T=kB*T/2./pi/h;
	double var=1.e-1*((freq_i>freq_T)*freq_T+(freq_i<=freq_T)*freq_i);
        for(double freq=freq_i;freq<=freq_i+2.e3*var;freq+=var)
        {
                res+=(1./6.)*(ion(i,freq)*plank(freq,T)
                +ion(i,freq+var)*plank(freq+var,T))
                +(2./3.)*ion(i,freq+var/2.)*plank(freq+var/2.,T);

        }
        res*=var*c;
        return res;
}

double rec_rate(double i, double T, double N_e)
{
      return ion_rate(i,T)/g(T)*i*i*N_e*exp(-T_Hyd(i)/T);
}

//---------------------------------------------------------------------
int main()
{
	return 0;
}
